@@ -8,6 +8,9 @@ from typing import cast
 
 import pytest
 
+from packaging.licenses import canonicalize_license_expression
+from packaging.utils import canonicalize_name
+
 from poetry.core.constraints.version import Version
 from poetry.core.constraints.version.exceptions import ParseConstraintError
 from poetry.core.factory import Factory
@@ -15,7 +18,8 @@ from poetry.core.packages.dependency import Dependency
 from poetry.core.packages.dependency_group import DependencyGroup
 from poetry.core.packages.package import Package
 from poetry.core.packages.project_package import ProjectPackage
-from poetry.core.version.exceptions import InvalidVersion
+from poetry.core.spdx.helpers import license_by_id
+from poetry.core.version.exceptions import InvalidVersionError
 
 
 if TYPE_CHECKING:
@@ -29,7 +33,7 @@ if TYPE_CHECKING:
 def package_with_groups() -> Package:
     package = Package("foo", "1.2.3")
 
-    optional_group = DependencyGroup("optional", optional=True)
+    optional_group = DependencyGroup("Optional", optional=True)
     optional_group.add_dependency(Factory.create_dependency("bam", "^3.0.0"))
 
     package.add_dependency(Factory.create_dependency("bar", "^1.0.0"))
@@ -43,11 +47,11 @@ def package_with_groups() -> Package:
 def test_package_authors() -> None:
     package = Package("foo", "0.1.0")
 
-    package.authors.append("Sébastien Eustace <sebastien@eustace.io>")
+    package.authors = ["Sébastien Eustace <sebastien@eustace.io>"]
     assert package.author_name == "Sébastien Eustace"
     assert package.author_email == "sebastien@eustace.io"
 
-    package.authors.insert(0, "John Doe")
+    package.authors = ["John Doe", *package.authors]
     assert package.author_name == "John Doe"
     assert package.author_email is None
 
@@ -55,9 +59,9 @@ def test_package_authors() -> None:
 def test_package_authors_invalid() -> None:
     package = Package("foo", "0.1.0")
 
-    package.authors.insert(0, "<John Doe")
+    package.authors = ["<John Doe"]
     with pytest.raises(ValueError) as e:
-        package.author_name
+        package.author_name  # noqa: B018
 
     assert (
         str(e.value)
@@ -73,7 +77,7 @@ def test_package_authors_invalid() -> None:
         ("'Jane Doe'", None),
         ('"Jane Doe"', None),
         ("MyCompany", None),
-        ("Some Company’s", None),
+        ("Some Company’s", None),  # noqa: RUF001
         ("MyCompany's R&D", "rnd@MyCompanyName.MyTLD"),
         ("Doe, John", None),
         ("(Doe, John)", None),
@@ -83,16 +87,15 @@ def test_package_authors_invalid() -> None:
         ("John-Paul: Doe", None),
         ("John-Paul: Doe", "jp@nomail.none"),
         ("John Doe the 3rd", "3rd@jd.net"),
+        ("FirstName LastName firstname.lastname@company.com", None),
+        ("Surname, Given Name [Department]", None),
     ],
 )
 def test_package_authors_valid(name: str, email: str | None) -> None:
     package = Package("foo", "0.1.0")
 
-    if email is None:
-        author = name
-    else:
-        author = f"{name} <{email}>"
-    package.authors.insert(0, author)
+    author = name if email is None else f"{name} <{email}>"
+    package.authors = [author]
     assert package.author_name == name
     assert package.author_email == email
 
@@ -101,20 +104,38 @@ def test_package_authors_valid(name: str, email: str | None) -> None:
     ("name",),
     [
         ("<john@john.doe>",),
-        ("john@john.doe",),
+        ("<john@john.doe",),
+        ("john@john.doe>",),
         ("<John Doe",),
-        ("John? Doe",),
-        ("Jane+Doe",),
-        ("~John Doe",),
-        ("John~Doe",),
     ],
 )
 def test_package_author_names_invalid(name: str) -> None:
     package = Package("foo", "0.1.0")
 
-    package.authors.insert(0, name)
+    package.authors = [name]
     with pytest.raises(ValueError):
-        package.author_name
+        package.author_name  # noqa: B018
+
+
+@pytest.mark.parametrize(
+    "license",
+    [None, "MIT", "Apache-2.0 OR BSD-2-clause", "LicenseRef-MyProprietaryLicense"],
+)
+def test_all_classifiers_no_license_classifiers_if_spdx(license: str) -> None:
+    package = Package("foo", "0.1.0")
+    if license:
+        package.license_expression = canonicalize_license_expression(license)
+
+    assert not any(c.lower().startswith("license") for c in package.all_classifiers)
+
+
+def test_all_classifiers_with_license_classifiers_if_no_spdx() -> None:
+    package = Package("foo", "0.1.0")
+    package.license = license_by_id("MIT")
+
+    assert [c for c in package.all_classifiers if c.lower().startswith("license")] == [
+        "License :: OSI Approved :: MIT License"
+    ]
 
 
 @pytest.mark.parametrize("groups", [["main"], ["dev"]])
@@ -371,6 +392,7 @@ def test_to_dependency_for_file() -> None:
         "1.2.3",
         source_type="file",
         source_url=path.as_posix(),
+        source_subdirectory="qux",
         features=["baz", "bar"],
     )
     dep = package.to_dependency()
@@ -383,6 +405,7 @@ def test_to_dependency_for_file() -> None:
     assert dep.path == path
     assert dep.source_type == "file"
     assert dep.source_url == path.as_posix()
+    assert dep.source_subdirectory == "qux"
 
 
 def test_to_dependency_for_url() -> None:
@@ -481,6 +504,20 @@ def test_without_dependency_groups(package_with_groups: Package) -> None:
     assert len(package.all_requires) == 2
 
 
+def test_without_dependency_groups_uses_normalized_names(
+    package_with_groups: Package,
+) -> None:
+    package = package_with_groups.without_dependency_groups(["DeV"])
+
+    assert len(package.requires) == 2
+    assert len(package.all_requires) == 3
+
+    package = package_with_groups.without_dependency_groups(["DeV", "OpTioNal"])
+
+    assert len(package.requires) == 2
+    assert len(package.all_requires) == 2
+
+
 def test_with_dependency_groups(package_with_groups: Package) -> None:
     package = package_with_groups.with_dependency_groups([])
 
@@ -493,6 +530,15 @@ def test_with_dependency_groups(package_with_groups: Package) -> None:
     assert len(package.all_requires) == 4
 
 
+def test_with_dependency_groups_uses_normalized_names(
+    package_with_groups: Package,
+) -> None:
+    package = package_with_groups.with_dependency_groups(["OpTioNal"])
+
+    assert len(package.requires) == 2
+    assert len(package.all_requires) == 4
+
+
 def test_without_optional_dependency_groups(package_with_groups: Package) -> None:
     package = package_with_groups.without_optional_dependency_groups()
 
@@ -517,25 +563,6 @@ def test_only_with_dependency_groups(package_with_groups: Package) -> None:
     assert len(package.all_requires) == 2
 
 
-def test_get_readme_property_with_multiple_readme_files() -> None:
-    package = Package("foo", "0.1.0")
-
-    package.readmes = (Path("README.md"), Path("HISTORY.md"))
-    with pytest.deprecated_call():
-        assert package.readme == Path("README.md")
-
-
-def test_set_readme_property() -> None:
-    package = Package("foo", "0.1.0")
-
-    with pytest.deprecated_call():
-        package.readme = Path("README.md")
-
-    assert package.readmes == (Path("README.md"),)
-    with pytest.deprecated_call():
-        assert package.readme == Path("README.md")
-
-
 @pytest.mark.parametrize(
     ("package", "dependency", "ignore_source_type", "result"),
     [
@@ -624,10 +651,11 @@ def test_python_versions_are_made_precise() -> None:
     package = Package("foo", "1.2.3")
     package.python_versions = ">3.6,<=3.10"
 
-    assert (
-        str(package.python_marker)
-        == 'python_full_version > "3.6.0" and python_full_version <= "3.10.0"'
-    )
+    with pytest.warns(DeprecationWarning):
+        assert (
+            str(package.python_marker)
+            == 'python_full_version > "3.6.0" and python_full_version <= "3.10.0"'
+        )
     assert str(package.python_constraint) == ">3.6,<=3.10"
 
 
@@ -639,7 +667,7 @@ def test_cannot_update_package_version() -> None:
 
 def test_project_package_version_update_string() -> None:
     package = ProjectPackage("foo", "1.2.3")
-    package.version = "1.2.4"  # type: ignore[assignment]
+    package.version = "1.2.4"
     assert package.version.text == "1.2.4"
 
 
@@ -664,7 +692,7 @@ def test_project_package_hash_not_changed_when_version_is_changed() -> None:
 
 
 def test_package_invalid_version() -> None:
-    with pytest.raises(InvalidVersion) as exc_info:
+    with pytest.raises(InvalidVersionError) as exc_info:
         Package("foo", "1.2.3.bogus")
 
     expected = "Invalid version '1.2.3.bogus' on package foo"
@@ -678,3 +706,63 @@ def test_package_invalid_python_versions() -> None:
 
     expected = "Invalid python versions '>=3.6.y' on foo (1.2.3)"
     assert str(exc_info.value) == expected
+
+
+def test_package_empty_python_versions() -> None:
+    package = Package("foo", "1.2.3")
+    with pytest.raises(ParseConstraintError) as exc_info:
+        package.python_versions = "~2.7, >=3.4, <3.8"
+
+    expected = "Python versions '~2.7, >=3.4, <3.8' on foo (1.2.3) is empty"
+    assert str(exc_info.value) == expected
+
+
+def test_package_dependency_group_names(package_with_groups: Package) -> None:
+    assert package_with_groups.dependency_group_names() == {"main", "dev"}
+    assert package_with_groups.dependency_group_names(include_optional=True) == {
+        "main",
+        "dev",
+        "optional",
+    }
+
+
+@pytest.mark.parametrize(
+    ("group_name", "expected"),
+    [("optional", True), ("Optional", True), ("OpTiOnAl", True), ("other", False)],
+)
+def test_package_has_dependency_group(
+    package_with_groups: Package, group_name: str, expected: bool
+) -> None:
+    assert package_with_groups.has_dependency_group(group_name) is expected
+
+
+@pytest.mark.parametrize("group_name", ["optional", "Optional", "OpTiOnAl"])
+def test_package_get_dependency_group(
+    package_with_groups: Package, group_name: str
+) -> None:
+    group = package_with_groups.dependency_group(group_name)
+
+    assert group.name == "optional"
+    assert group.pretty_name == "Optional"
+
+
+@pytest.mark.parametrize("group_name", ["optional", "Optional", "OpTiOnAl"])
+def test_package_add_to_dependency_group(
+    package_with_groups: Package, group_name: str
+) -> None:
+    dependency = Dependency("foo-bar", "^1.0.0", groups=[group_name])
+    normalized_name = canonicalize_name(group_name)
+
+    assert dependency not in package_with_groups.all_requires
+    assert (
+        dependency
+        not in package_with_groups._dependency_groups[normalized_name].dependencies
+    )
+
+    package_with_groups.add_dependency(dependency)
+
+    assert dependency in package_with_groups.all_requires
+    assert (
+        dependency
+        in package_with_groups._dependency_groups[normalized_name].dependencies
+    )
